/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : May, 1997                                                      */
/* File Name : List.cc                                                   */
/*************************************************************************/
/*
   This object implements the List objects necessary functionality.
*/
/*************************************************************************/

#include <stdio.h>
#include "List.h"
#include "ListNode.h"

/*
// Creates an empty List.
*/

template<class T1>
List<T1>::List() 
{  head = NULL;
   size = 0;
}// end of constructor

/*
//
// Copy Constructor.
//
// 1. Add each element of the Right hand side list.
//
*/

template<class T1>
List<T1>::List(const List<T1>& RHS)
{  
   head = NULL;
   size = 0;

   // Copy the RHS List

   copyFrom(RHS);

}// end of COPY constructor

/*
// Deallocate the current list.
*/

template<class T1>
List<T1>::~List() 
{  
   dispose();

}// end of destructor

/*
// Adds from the beginning of the List.
*/

template<class T1>
void List<T1>::addElement(T1 flg) 
{  
   ListNode<T1>* newEle = new ListNode<T1>(flg);
   newEle->setNext(head);
   head = newEle;
   size++;

}// end of addElement()

/*
// Removes element from the List.
*/

template<class T1>
void List<T1>::removeElement(T1 flg) 
{  
   ListNode<T1>* next = head;
   ListNode<T1>* prev = head;
   int found = 0;

   // find the element
   while( !found && next != NULL)
   {
     if( flg == next->getValue() ) found = 1;
     else
      {  prev = next;
	 next = next->getNext();
      }

   }// end of while

   // delete the element if it is found
   if(found)
   {
     // Update the links
     prev = next->getNext();

     // delete the element
     delete next;

   }// end of if

}// end of removeElement()

/*
// Gets the number of elements in the List.
*/

template<class T1> 
unsigned List<T1>::getSize() const
{ return size; }

/*
// Gets the element at given Index.
*/

template<class T1>
T1 List<T1>::getElement(int indx)
{  T1 ret;
   ListNode<T1>* next = head;
   unsigned i=0;

   while ( i<indx && next!=NULL) {  next = next->getNext(); i++; }

   if(next!=NULL) ret = next->getValue();

   return ret; 

}// end of getElement()

/*
// Assignment Operation.
*/

template<class T1>
List<T1> List<T1>::operator=(const List& RHS)
{

   dispose();
   copyFrom(RHS);

   return (*this);

}// end of operator=()

/*
// Compares two List objects.
*/

template<class T1>
List<T1>::operator==(const List& RHS)
{  int ret=1;
   int stay=1;

   ret = ret && ( (head != NULL && RHS.head !=NULL)  
                  || (head == NULL && RHS.head ==NULL) );  

   if(ret && head != NULL && RHS.head !=NULL)
   {  ListNode<T1>* next = head;
      ListNode<T1>* RHSnext = RHS.head;

      while(ret && stay)
      {  ret = ret && ( *next == *RHSnext );
	 if(next->getNext()!=NULL && RHSnext->getNext()!=NULL)
	 {  next = next->getNext();
	    RHSnext = RHSnext->getNext();
	 }
	 else if(next->getNext()==NULL && RHSnext->getNext()==NULL) stay = 0;
	 else ret = 0;

      }// end of while 

   }// end of if

   return ret;

}// end of operator==()

/*
// Dispose the List
*/

template <class T1>
void List<T1>::dispose()
{
   ListNode<T1>* next = head;

   while(head!=NULL)
   { next = head->getNext();
     delete head;
     head = next;
   }// end of while

   head = NULL;
   size = 0;

}// end of dispose()

/*
// Copy theList from another List
*/

template <class T1>
void List<T1>::copyFrom(const List<T1>& RHS)
{
   ListNode<T1>* RHShead=RHS.head;

   // Copy the RHS List

   while(RHShead!=NULL)
   {  addElement(RHShead->getValue());
      RHShead = RHShead->getNext();
   }// end of while

}// end of copyFrom()

/*
// Prints the List information.
*/

template<class T1>
void List<T1>::printMe(ostream& os) const
{
   ListNode<T1>* first = head;

   while( first != NULL )
   { os << " " << (*first) ;
     first = first->getNext();
   }// end of while

   os << " ";

}// end of printMe()

/*
// Prints the List to the output stream.
*/

template <class T1>
ostream & operator << ( ostream& os, const List<T1>& RHS)
{
   RHS.printMe(os);
   return os;

}// end of operator<<()




